@@ -4,13 +4,18 @@ from typing import List
 import six
 from time import time
 
+try:
+    import cPickle as pickle
+except ImportError:  # pragma: nocover
+    import pickle
+
 from uhashring import HashRing
-from fastapi_crawler_scheduler.apscheduler.jobstores.redis import RedisJobStore
-from fastapi_crawler_scheduler.apscheduler.schedulers.background import BackgroundScheduler
-from fastapi_crawler_scheduler.apscheduler.util import ref_to_obj
+from apscheduler.jobstores.redis import RedisJobStore
+from apscheduler.schedulers.background import BackgroundScheduler
+from apscheduler.util import ref_to_obj, undefined, datetime_to_utc_timestamp
+from apscheduler.job import Job
 
 from fastapi_crawler_scheduler.service.dbRedisHelper import DbRedisHelper, standard_time
-from fastapi_crawler_scheduler.utils.exception import SchedulerError
 
 setp = 1
 
@@ -67,22 +72,44 @@ class BaseScheduler(object):
         interval = crawler_info.get('interval')
         trigger = crawler_info.get('trigger')
         job_id = crawler_info.get('job_id')
-        if self.redis_job_store.redis.hexists(self.redis_job_store.jobs_key, job_id):
-            self.redis_job_store.remove_job(job_id=job_id)
+        # if self.redis_job_store.redis.hexists(self.redis_job_store.jobs_key, job_id):
+        #     self.redis_job_store.remove_job(job_id=job_id)
         crawler_info = format_params(params=crawler_info)
-        # try:
-        self.scheduler.add_job(
-            func,
-            id=job_id,
-            trigger=trigger,
-            kwargs=crawler_info,
-            seconds=interval,
-        )
-        # except Exception as e:
-        #     # print(e)
-        #     # print("scheduler   add_job 错误")
-        #     # raise Exception
-        #     raise SchedulerError(f"scheduler   add_job 错误 {e}")
+        job_kwargs = {
+            'trigger': self.scheduler._create_trigger(trigger, dict(seconds=interval)),
+            'executor': 'default',
+            'func': func,
+            'args': (),
+            'kwargs': crawler_info,
+            'id': job_id,
+            'name': None,
+            'misfire_grace_time': undefined,
+            'coalesce': undefined,
+            'max_instances': undefined,
+            'next_run_time': undefined
+        }
+        job_kwargs = dict((key, value) for key, value in six.iteritems(job_kwargs) if value is not undefined)
+        job = Job(self, **job_kwargs)
+        if self.redis_job_store.redis.hexists(self.redis_job_store.jobs_key, job.id):
+            self.redis_job_store.remove_job(job_id=job_id)
+
+        with self.redis_job_store.redis.pipeline() as pipe:
+            pipe.multi()
+            pipe.hset(self.redis_job_store.jobs_key, job.id, pickle.dumps(job.__getstate__(),
+                                                                          self.redis_job_store.pickle_protocol))
+            if job.next_run_time:
+                pipe.zadd(self.redis_job_store.run_times_key,
+                          {job.id: datetime_to_utc_timestamp(job.next_run_time)})
+
+            pipe.execute()
+
+        # self.scheduler.add_job(
+        #     func,
+        #     id=job_id,
+        #     trigger=trigger,
+        #     kwargs=crawler_info,
+        #     seconds=interval,
+        # )
 
     def check_process(self) -> None:
         self.redis_db.process_acquire(f'{self.project_name}:node:{self.uuid_number}:{os.getpid()}')
